@@ -32,17 +32,21 @@ sub _guard_on_die : prototype(&$) {
   my ($want, $res, @res) = (wantarray);
 
   # perltidy doesn't allow to rewrite this in 3 lines.
-  local $@;
-  if (!defined $want) {
-    eval { $cb->() };
+  my $err;
+CATCH: {
+    local $@;
+    if (!defined $want) {
+      eval { $cb->() };
+    }
+    elsif ($want) {
+      @res = eval { $cb->() };
+    }
+    else {
+      $res = eval { $cb->() };
+    }
+    $err = $@;
   }
-  elsif ($want) {
-    @res = eval { $cb->() };
-  }
-  else {
-    $res = eval { $cb->() };
-  }
-  do { $die->(); Carp::confess $@ } if $@;
+  do { $die->(); Carp::confess $err } if $err;
 
   return unless defined $want;
   $want ? @res : $res;
@@ -55,19 +59,22 @@ sub _guard : prototype(&$) {
   my ($want, $res, @res) = (wantarray);
 
   # perltidy doesn't allow to rewrite this in 3 lines.
-  local $@;
-  if (!defined $want) {
-    eval { $cb->() };
+  my $err;
+CATCH: {
+    local $@;
+    if (!defined $want) {
+      eval { $cb->() };
+    }
+    elsif ($want) {
+      @res = eval { $cb->() };
+    }
+    else {
+      $res = eval { $cb->() };
+    }
+    $err = $@;
   }
-  elsif ($want) {
-    @res = eval { $cb->() };
-  }
-  else {
-    $res = eval { $cb->() };
-  }
-
   $always->();
-  do { Carp::confess $@ } if $@;
+  do { Carp::confess $err } if $err;
 
   return unless defined $want;
   $want ? @res : $res;
@@ -96,7 +103,7 @@ Evo::Util
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 DESCRIPTION
 
